Unit Trigo;

{ Dragon's Fly - Copyright (C) 2K by Paul TOTH <tothpaul@free.fr>
 http://tothpaul.free.fr

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

}

interface

{$N+}

Type
 TAngle=byte;

Const
 A180=128;
 A90 =64;
 A45 =32;
 A30 =21;

Type
 TVertice=record
  x,y,z:integer;
 end;

 TFace=record
  a,b,c:word;
 end;

 TCoord   =record x,y,z:integer end;
 TCoords  =array[0..1000] of TCoord;

 TFaceList=array[0..1000] of word;
 TDeltaz  =array[0..1000] of longint;

 TVector  =TCoord;
 TVectors =array[0..1000] of TVector;

var
 Coords   :^TCoords;
 TempNorm :^TVectors;
 TempNorm2:^TVectors;

 Light    :TVector;

 FaceList :^TFaceList;
 DeltaZ   :^TDeltaZ;

 ticks:longint absolute $40:$6C;
 time :longint;
 count:word;

procedure FaceNormals(Var Faces:array of TFace; var Vertices,FNormals:array of TVertice; Normal:word);
procedure Normals(Var Faces:array of TFace;
                  var Vertices,FNormals,VNormals:array of TVertice;
                  Normal:word);

procedure InitTrigo(Phix,Phiy,Phiz:word);
procedure translate(x,y,z:integer;src,dst:pointer;count:word);
procedure rotate(src,dst:pointer;count:word);
procedure project(dst:pointer; count:word);


implementation

const
 FACTOR=1 SHL 8;

 CENTER_Z=2000;
 CENTER_X=160;
 CENTER_Y=100;

var
 sintab,costab:array[0..255] of integer;
 sinx,siny,sinz:integer;
 cosx,cosy,cosz:integer;
 rx,ry,rz:word;

procedure FaceNormals(Var Faces:array of TFace; var Vertices,FNormals:array of TVertice; Normal:word);
 var
  i:word;
  x1,y1,z1,x2,y2,z2,vl:single;
  nx,ny,nz:single;
 begin
 { Face Normals }
  For i:=Low(Faces) To High(Faces) do begin
   with Faces[i] do begin
    x1:=Vertices[b].x-Vertices[a].x;
    y1:=Vertices[b].y-Vertices[a].y;
    z1:=Vertices[b].z-Vertices[a].z;
    x2:=Vertices[c].x-Vertices[a].x;
    y2:=Vertices[c].y-Vertices[a].y;
    z2:=Vertices[c].z-Vertices[a].z;
   end;
   nx:=y1*z2-y2*z1;
   ny:=z1*x2-z2*x1;
   nz:=x1*y2-x2*y1;
   vl:=sqrt(nx*nx+ny*ny+nz*nz);
   if vl=0 then vl:=1;
   with FNormals[i] do begin
    x:=Round((nx/vl)*Normal);
    y:=Round((ny/vl)*Normal);
    z:=Round((nz/vl)*Normal);
   end;
  end;
 end;

procedure Normals(Var Faces:array of TFace;var Vertices,FNormals,VNormals:array of TVertice;Normal:word);
 var
  i,j,nf:word;
  x1,y1,z1,x2,y2,z2,vl:single;
  nx,ny,nz:single;
 begin
 { Face Normals }
  For i:=Low(Faces) To High(Faces) do begin
   with Faces[i] do begin
    x1:=Vertices[b].x-Vertices[a].x;
    y1:=Vertices[b].y-Vertices[a].y;
    z1:=Vertices[b].z-Vertices[a].z;
    x2:=Vertices[c].x-Vertices[a].x;
    y2:=Vertices[c].y-Vertices[a].y;
    z2:=Vertices[c].z-Vertices[a].z;
   end;
   nx:=y1*z2-y2*z1;
   ny:=z1*x2-z2*x1;
   nz:=x1*y2-x2*y1;
   vl:=sqrt(nx*nx+ny*ny+nz*nz);
   if vl=0 then vl:=1;
   with FNormals[i] do begin
    x:=Round((nx/vl)*Normal);
    y:=Round((ny/vl)*Normal);
    z:=Round((nz/vl)*Normal);
   end;
  end;
 { Vertex Normals }
  for i:=Low(Vertices) to High(Vertices) do begin
   x1:=0; y1:=0; z1:=0; nf:=0;
   for j:=Low(Faces) to High(Faces) do begin
    with Faces[j] do if (a=i)or(b=i)or(c=i) then begin
     with FNormals[j] do begin
      x1:=x1+x; y1:=y1+y; z1:=z1+z;
     end;
     inc(nf);
    end;
   end;
   if nf<>0 then begin
    x1:=x1/nf; y1:=y1/nf; z1:=z1/nf;
    vl:=sqrt(x1*x1+y1*y1+z1*z1);
    with VNormals[i] do begin
     x:=Round((x1/vl)*Normal);
     y:=Round((y1/vl)*Normal);
     z:=Round((z1/vl)*Normal);
    end;
   end;
  end;
 end;

procedure InitTrigo(Phix,Phiy,Phiz:word); assembler;
 asm
  mov si,Offset SinTab
  mov di,Offset CosTab

  mov bx,PhiX; add bx,bx
  mov ax,[bx+si]; mov sinx,ax
  mov ax,[bx+di]; mov cosx,ax

  mov bx,PhiY; add bx,bx
  mov ax,[bx+si]; mov siny,ax
  mov ax,[bx+di]; mov cosy,ax

  mov bx,PhiZ; add bx,bx
  mov ax,[bx+si]; mov sinz,ax
  mov ax,[bx+di]; mov cosz,ax
 end;

procedure translate(x,y,z:integer;src,dst:pointer;count:word); assembler;
 asm
 push ds
  les di,dst
  mov bx,x
  mov cx,y
  mov dx,z
  lds si,src
  @next_point:

   lodsw; add ax,bx; stosw
   lodsw; add ax,cx; stosw
   lodsw; add ax,dx; stosw

  dec count
  jnz @next_point
 pop ds
 end;

procedure rotate(src,dst:pointer;count:word); assembler;
 asm
 { rotate }

  mov si,word(src)
  mov di,word(dst)

  mov cx,count
  @next_point:

    { get point }
    mov es,word(src+2)
    mov ax,es:[si+0]; mov rx,ax
    mov ax,es:[si+2]; mov ry,ax
    mov ax,es:[si+4]; mov rz,ax

    { rotate x : y'=y*cosx-z*sinx / z'=y*sinx+z*cosx}
    mov ax,rz
    imul sinx
    mov bl,ah; mov bh,dl

    mov ax,ry
    imul cosx
    mov al,ah; mov ah,dl
    sub ax,bx
    xchg ax,ry

    imul sinx
    mov bl,ah; mov bh,dl
    mov ax,rz
    imul cosx
    mov al,ah; mov ah,dl
    add ax,bx
    mov rz,ax

    { rotate y : x'=x*cosy+z*siny / z'=z*coxy-x*siny}
    mov ax,rx
   imul cosy
    mov bl,ah; mov bh,dl
    mov ax,rz
   imul siny
    mov al,ah; mov ah,dl
    add ax,bx
    xchg rx,ax

   imul siny
    mov bl,ah; mov bh,dl
    mov ax,rz
   imul cosy
    mov al,ah; mov ah,dl
    sub ax,bx
    mov rz,ax

    { rotate z : x'=x*cosz-y*sinz / y'=x*sinz+y*cosz }
    mov ax,ry
   imul sinz
    mov bl,ah; mov bh,dl
    mov ax,rx
   imul cosz
    mov al,ah; mov ah,dl
    sub ax,bx
    xchg ax,rx

   imul sinz
    mov bl,ah; mov bh,dl
    mov ax,ry
   imul cosz
    mov al,ah; mov ah,dl
    add ax,bx
    mov ry,ax

    { set point }
    mov es,word(dst+2)
    mov ax,rx; mov es:[di+0],ax;
    mov ax,ry; mov es:[di+2],ax;
    mov ax,rz; mov es:[di+4],ax;

    add si,6
    add di,6

   dec cx
   jnz @next_point
 (*  loop @next_point *)
 end;

procedure project(dst:pointer; count:word); assembler;
 asm
  les di,dst
  mov cx,Count
  @next_point:

   mov bx,es:[di+4]
   add bx,CENTER_Z

   mov ax,es:[di]; cwd; mov dl,ah; mov ah,al; mov al,0
  idiv bx
   add ax,CENTER_X
   mov es:[di],ax
   add di,2

   mov ax,es:[di]; cwd; mov dl,ah; mov ah,al; mov al,0
  idiv bx
   mov bx,CENTER_Y
   sub bx,ax
   mov es:[di],bx
   add di,4

  loop @next_point
 end;

var
 x:byte;
begin
 for x:=0 to 255 do begin
  sintab[x]:=round(sin(x*PI/128)*FACTOR);
  costab[x]:=round(cos(x*PI/128)*FACTOR);
 end;
end.