unit Dragon1;

{ Dragon's Fly - Copyright (C) 2K by Paul TOTH <tothpaul@free.fr>
 http://tothpaul.free.fr

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls;

type
  TForm1 = class(TForm)
    Timer1: TTimer;
    Image1: TImage;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
  private
    { Dclarations prives }
    procedure Compute;
    procedure Fly(var msg:TMessage); message wm_user;
  public
    { Dclarations publiques }
  end;

var
  Form1: TForm1;

implementation

Uses Trigo; (* delphi version of an old Pascal unit :) *)

{$R *.DFM}

Const
 Age   =40; (* Dragon's Age = number of blocks *)

 dWidth =80; (* block size *)
 dHeight=60;
 dDepth =50;

 Size:array[1..3*8] of integer=(
  -1,-1,+1,
  +1,-1,+1,
  -1,+1,+1,
  +1,+1,+1,
  -1,-1,-1,
  +1,-1,-1,
  -1,+1,-1,
  +1,+1,-1
 );

 Edge:array[0..3*12-1] of word=(
  0,2,1, 1,2,3,
  4,5,6, 5,7,6,
  0,1,4, 4,1,5,
  6,7,2, 2,7,3,
  0,4,6, 6,2,0,
  7,5,1, 7,1,3
 );

 VerticeCount=Age*8;
 FaceCount   =Age*12;

Var
 Vertices:array[0..VerticeCount-1] of TVertice;
 Coords  :array[0..VerticeCount-1] of TCoord;
 Faces   :array[0..FaceCount-1] of TFace;
 FNormals:array[0..FaceCount-1] of TVertice;
 TempNorm:array[0..FaceCount-1] of TVector;

 FaceList:array[0..FaceCount-1] of integer;
 DeltaZ  :array[0..FaceCount-1] of longint;

 d,dv,df,v,f:integer;
 vc,fc:integer;

 mx,my:integer;

 phix,phiy,phiz:byte;
 phia,phib,phic:byte;
 da,db,dc:integer;

procedure QuickSort(left,right:word);
 var
  i,j:word;
  x,y:longint;
 begin
  i:=left;
  j:=right;
  x:=deltaz[(left+right+1)shr 1];
  repeat
   while (deltaz[i]>x) do inc(i);
   while (x>deltaz[j]) do dec(j);
   if (i<=j) then begin
    y:=deltaz[i]; deltaz[i]:=deltaz[j]; deltaz[j]:=y;
    y:=facelist[i]; facelist[i]:=facelist[j]; facelist[j]:=y;
    inc(i);
    dec(j);
   end;
  until i>j;
  if (left<j) then quicksort(left,j);
  if (i<right) then quicksort(i,right);
 end;

function Shade(const vector:TVector):integer;
 begin
  Shade:=32+((Light.x*Vector.x)+(Light.y*Vector.y)+(Light.z*Vector.z*3 div 4));
 end;

function Faced(const vector:TVector):boolean;
 begin
  with vector do Faced:=(Light.x*x+Light.y*y+Light.z*z)>=0
 end;

procedure TForm1.FormCreate(Sender: TObject);
begin
{ Build Dragon Vertices & Faces }
 for d:=0 to Age-1 do begin

  dv:=8*d;
  Move(Size,Vertices[dv],SizeOf(Size));
  for v:=0 to 7 do begin
   With Vertices[dv+v] do begin
    x:=x*(dWidth -2*d);
    y:=y*(dHeight-2*d);
    z:=z*(dDepth -2*d)+dDepth;
   end;
  end;

  df:=12*d;
  Move(Edge,Faces[df],SizeOf(Edge));
  for f:=0 to 11 do begin
   With Faces[df+f] do begin
    inc(a,dv);
    inc(b,dv);
    inc(c,dv);
   end;
  end;

 end;

(* compute normals *) 
 FaceNormals(Faces,Vertices,FNormals,32);

 with Light do begin x:=0; y:=0; z:=1; end;

 phix:=0;
 phiy:=32;
 phiz:=0;

 phia:=0; da:=1;
 phib:=0; db:=1;
 phic:=5; dc:=1;

(* start the fly :) *)
 postmessage(handle,wm_user,0,0);
end;

procedure TForm1.Compute;
begin
(* get a copy of all vertices & normals *)
 Move(Vertices,Coords,SizeOf(Coords));
 Move(FNormals,TempNorm,SizeOf(FNormals));
 vc:=0;
 fc:=0;

 if phia<16 then begin
  if phic<5 then inc(phic)
 end else begin
  if phic>0 then dec(phic);
 end;

(* for each block *)
 for d:=Age-1 downto 0 do begin
  dv:=8*d;
  df:=12*d;
  inc(vc,8);
  inc(fc,12);
  { Rotate Age# and following ones }
  InitTrigo(phia,phib,phic);
  Rotate(@Coords[dv],@Coords[dv],vc);
  Rotate(@TempNorm[df],@TempNorm[df],fc);
  { Translate them }
  if d>0 then Translate(0,0,2*dDepth-3*d,@Coords[dv],@Coords[dv],vc);
 end;
(* final transformation *)
 InitTrigo(PhiX,PhiY,PhiZ);
 Rotate(@Coords,@Coords,VerticeCount);
 Translate(mx,my,-500,@Coords,@Coords,VerticeCount);
 Rotate(@TempNorm,@TempNorm,FaceCount);
(* and now compute screen coordonates *)
 Project(@Coords,VerticeCount);
end;

procedure TForm1.Fly(Var Msg:TMessage);
 var
  r:TRect;
  i:word;
  s:word;
  cl:TColor;
begin
 repeat
 Compute; (* compute 2D points *)

 r:=Image1.ClientRect;
 mx:=r.Right;
 my:=r.Bottom;

 with Image1.Canvas do begin

 Brush.Color:=0;
 FillRect(r);

 for i:=0 to FaceCount-1 do begin
  FaceList[i]:=i;
  with Faces[i] do deltaZ[i]:=longint(Coords[a].z+Coords[b].z+Coords[c].z) div 3;
 end;

 QuickSort(0,FaceCount-1);

 for i:=0 to FaceCount-1 do begin
  with Faces[FaceList[i]] do begin
   If (*Faced*)(TempNorm[FaceList[i]].z>=0) then
   s:=Shade(TempNorm[FaceList[i]]);
   cl:=s shl 16+s shl 8+s shl 2;
   Pen.Color:=cl;
   Brush.Color:=cl;
   Polygon([Point(coords[a].x,coords[a].y),
            Point(coords[b].x,coords[b].y),
            Point(coords[c].x,coords[c].y)]);
  end;
 end;
 end;

 inc(phia,da); if (phia>16)and(phia<240) then da:=-da;
 inc(phix,2);
 application.processmessages;
 until application.Terminated;
end;

end.
