Unit Trigo;

{ Dragon's Fly - Copyright (C) 2K by Paul TOTH <tothpaul@free.fr>
 http://tothpaul.free.fr

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

}

interface

{$N+}

Type
 Integer=SmallInt;
 TAngle=byte;

Const
 A180=128;
 A90 =64;
 A45 =32;
 A30 =21;

Type
 TVertice=record
  x,y,z:integer;
 end;

 TFace=record
  a,b,c:word;
 end;

 TCoord   =record x,y,z:integer end;
 TCoords  =array[0..1000] of TCoord;

 TFaceList=array[0..1000] of word;
 TDeltaz  =array[0..1000] of longint;

 TVector  =TCoord;
 TVectors =array[0..1000] of TVector;

var
 Coords   :^TCoords;
 TempNorm :^TVectors;
 TempNorm2:^TVectors;

 Light    :TVector;

 FaceList :^TFaceList;
 DeltaZ   :^TDeltaZ;

 time :longint;
 count:word;

procedure FaceNormals(Var Faces:array of TFace; var Vertices,FNormals:array of TVertice; Normal:word);
procedure Normals(Var Faces:array of TFace; var Vertices,FNormals,VNormals:array of TVertice; Normal:word);

procedure InitTrigo(Phix,Phiy,Phiz:word);
procedure translate(x,y,z:integer;src,dst:pointer;count:word);
procedure rotate(src,dst:pointer;count:word);
procedure project(dst:pointer; count:word);


implementation

const
 FACTOR=1 SHL 8;

 CENTER_Z=2000;
 CENTER_X=160;
 CENTER_Y=100;

var
 sintab,costab:array[0..255] of integer;
 sinx,siny,sinz:integer;
 cosx,cosy,cosz:integer;
 rx,ry,rz:word;

procedure FaceNormals(Var Faces:array of TFace; var Vertices,FNormals:array of TVertice; Normal:word);
 var
  i:word;
  x1,y1,z1,x2,y2,z2,vl:single;
  nx,ny,nz:single;
 begin
 { Face Normals }
  For i:=Low(Faces) To High(Faces) do begin
   with Faces[i] do begin
    x1:=Vertices[b].x-Vertices[a].x;
    y1:=Vertices[b].y-Vertices[a].y;
    z1:=Vertices[b].z-Vertices[a].z;
    x2:=Vertices[c].x-Vertices[a].x;
    y2:=Vertices[c].y-Vertices[a].y;
    z2:=Vertices[c].z-Vertices[a].z;
   end;
   nx:=y1*z2-y2*z1;
   ny:=z1*x2-z2*x1;
   nz:=x1*y2-x2*y1;
   vl:=sqrt(nx*nx+ny*ny+nz*nz);
   if vl=0 then vl:=1;
   with FNormals[i] do begin
    x:=Round((nx/vl)*Normal);
    y:=Round((ny/vl)*Normal);
    z:=Round((nz/vl)*Normal);
   end;
  end;
 end;

procedure Normals(Var Faces:array of TFace;var Vertices,FNormals,VNormals:array of TVertice;Normal:word);
 var
  i,j,nf:word;
  x1,y1,z1,x2,y2,z2,vl:single;
  nx,ny,nz:single;
 begin
 { Face Normals }
  For i:=Low(Faces) To High(Faces) do begin
   with Faces[i] do begin
    x1:=Vertices[b].x-Vertices[a].x;
    y1:=Vertices[b].y-Vertices[a].y;
    z1:=Vertices[b].z-Vertices[a].z;
    x2:=Vertices[c].x-Vertices[a].x;
    y2:=Vertices[c].y-Vertices[a].y;
    z2:=Vertices[c].z-Vertices[a].z;
   end;
   nx:=y1*z2-y2*z1;
   ny:=z1*x2-z2*x1;
   nz:=x1*y2-x2*y1;
   vl:=sqrt(nx*nx+ny*ny+nz*nz);
   with FNormals[i] do begin
    x:=Round((nx/vl)*Normal);
    y:=Round((ny/vl)*Normal);
    z:=Round((nz/vl)*Normal);
   end;
  end;
 { Vertex Normals }
  for i:=Low(Vertices) to High(Vertices) do begin
   x1:=0; y1:=0; z1:=0; nf:=0;
   for j:=Low(Faces) to High(Faces) do begin
    with Faces[j] do if (a=i)or(b=i)or(c=i) then begin
     with FNormals[j] do begin
      x1:=x1+x; y1:=y1+y; z1:=z1+z;
     end;
     inc(nf);
    end;
   end;
   if nf<>0 then begin
    x1:=x1/nf; y1:=y1/nf; z1:=z1/nf;
    vl:=sqrt(x1*x1+y1*y1+z1*z1);
    with VNormals[i] do begin
     x:=Round((x1/vl)*Normal);
     y:=Round((y1/vl)*Normal);
     z:=Round((z1/vl)*Normal);
    end;
   end;
  end;
 end;

procedure InitTrigo(Phix,Phiy,Phiz:word);
 begin
  sinx:=SinTab[PhiX];
  cosx:=CosTab[PhiX];
  siny:=SinTab[PhiY];
  cosy:=CosTab[PhiY];
  sinz:=SinTab[PhiZ];
  cosz:=CosTab[PhiZ];
 end;

procedure translate(x,y,z:integer;src,dst:pointer;count:word);
 var
  i:integer;
  p1,p2:^TVertice;
 begin
  p1:=src;
  p2:=dst;
  for i:=1 to count do begin
   p2.x:=p1.x+x;
   p2.y:=p1.y+y;
   p2.z:=p1.z+z;
   inc(p1);
   inc(p2);
  end;
 end;

procedure rotate(src,dst:pointer;count:word);
 var
  i:integer;
  p1,p2:^TVertice;
  x1,y1,z1:integer;
  x2,y2,z2:integer;
 begin
  p1:=src;
  p2:=dst;
  for i:=1 to count do begin
   x1:=p1.x;
   y1:=p1.y;
   z1:=P1.z;

  { rotate x }
   y2:=(y1*cosx) div Factor-(z1*sinx) div Factor;
   z2:=(y1*sinx) div Factor+(z1*cosx) div Factor;

  { rotate y }
   x2:=(x1*cosy) div Factor+(z2*siny) div Factor;
   z1:=(z2*cosy) div Factor-(x1*siny) div Factor;

  { rotate z }
   x1:=(x2*cosz) div Factor-(y2*sinz) div Factor;
   y1:=(x2*sinz) div Factor+(y2*cosz) div Factor;

   p2.x:=x1;
   p2.y:=y1;
   p2.z:=z1;

   inc(p1);
   inc(p2);                          
  end;
 end;

procedure project(dst:pointer; count:word);
 var
  i:integer;
  p:^TVertice;
  z:integer;
 begin
  p:=dst;
  for i:=1 to count do begin
   z:=p.z+CENTER_Z;
   p.x:=CENTER_X+(p.x*Factor) div z;
   p.y:=CENTER_Y+(p.y*Factor) div z;
   inc(p);
  end;
 end;


var
 x:byte;
begin
 for x:=0 to 255 do begin
  sintab[x]:=round(sin(x*PI/128)*FACTOR);
  costab[x]:=round(cos(x*PI/128)*FACTOR);
 end;
end.